use strict;
use warnings;

use Produto;
use Compra;
use Fornecedor;
use Venda;
use Cliente;

my @entrada = @ARGV;

my @produtos;
my @compras;
my @fornecedores;
my @vendas;
my @clientes;

my $arqProd;
my $arqCompra;
my $arqForn;
my $arqVenda;
my $arqCliente;


sub calcValorCompra{
	my $comp = $_[0];
	my $prod;
	my $soma = 0;
	foreach $prod (@produtos){
		if($prod->{codigo} == $comp->{codProduto}){
			$soma = $comp->{quantidade}*$prod->{custo};
		}
	}
	return $soma;
}

sub montaPrimRelatorio{
	foreach $a (@fornecedores){
		$a->{totalPagar} = 0;
		my $c;
		foreach $c (@compras){
			if($c->{codFornecedor} == $a->{codigo}){
				$a->{totalPagar} += calcValorCompra( $c );
			}
		}
	}

	my @saida1 = sort {$a->{nome} cmp $b->{nome}} @fornecedores;

	return @saida1;
}


sub calcValorVenda{
	my $vend = $_[0];
	my $prod;
	my $fator;
	my $soma = 0;
	foreach $prod (@produtos){
		if($prod->{codigo} == $vend->{codProduto}){
			$fator = 1 + ( $prod->{percentual} )/100;
			$soma = $vend->{quantidade}*$prod->{custo}*$fator;
		}
	}
	return $soma;
}

sub montaSegRelatorio{
	foreach $a (@clientes){
		$a->{totalReceber} = 0;
		my $c;
		foreach $c (@vendas){
			if($c->{codCliente} == $a->{codigo}){
				$a->{totalReceber} += calcValorVenda( $c );
			}
		}
	}

	my @saida2 = sort {$a->{nome} cmp $b->{nome}} @clientes;

	return @saida2;
}

sub geraSaida{
	open(my $saida1,'>',"1-apagar.csv");
	open(my $saida2,'>',"2-areceber.csv");

	print $saida1 "Fornecedor;CNPJ;Pessoa de contato ;Telefone;Total a pagar\n";
	print $saida2 "Cliente;Tipo;CPF/CNPJ;Telefone;Data de cadastro;Total a pagar\n";

	my @lista1 = montaPrimRelatorio;
	my @lista2 = montaSegRelatorio;

	my $valor;

	foreach $a (@lista1){
		print $saida1 $a->{nome}.";";
		print $saida1 $a->{cnpj}.";";
		print $saida1 $a->{contato}.";";
		print $saida1 $a->{telefone}.";";
		$valor = $a->{totalPagar};
		$valor = sprintf("%.02f",$valor);
		$valor =~ tr/./,/;
		print $saida1 'R$ '.$valor."\n";
	}

	foreach $a (@lista2){
		print $saida2 $a->{nome}.";";
		print $saida2 $a->{tipoCliente}.";";
		print $saida2 $a->{cadNacional}.";";
		print $saida2 $a->{telefone}.";";
		print $saida2 $a->{dataCadastro}.";";
		$valor = $a->{totalReceber};
		$valor = sprintf("%.02f",$valor);
		$valor =~ tr/./,/;
		print $saida2 'R$ '.$valor."\n";
	}

}


for(my $var = 0 ; $var < scalar(@entrada) ; ++$var){
	if ($entrada[$var] eq "-p") {open($arqProd, '<', $entrada[$var+1]) or die "Não foi possível abrir o arquivo de produtos.\n"};
	if ($entrada[$var] eq "-a") {open($arqCompra, '<', $entrada[$var+1]) or die "Não foi possível abrir o arquivo de compras.\n"};
	if ($entrada[$var] eq "-f") {open($arqForn, '<', $entrada[$var+1]) or die "Não foi possível abrir o arquivo de fornecedores.\n"};
	if ($entrada[$var] eq "-v") {open($arqVenda, '<', $entrada[$var+1]) or die "Não foi possível abrir o arquivo de vendas.\n"};
	if ($entrada[$var] eq "-c") {open($arqCliente, '<', $entrada[$var+1]) or die "Não foi possível abrir o arquivo de clientes.\n"}; 
}

my $text;
my $obj;

$text = <$arqProd>;
while($text = <$arqProd>){
	chomp($text);
	$text =~ tr/,/./;
	my @lines = split(";", $text);
	last if($text eq "");
	$obj = Produto->new(@lines);
	push @produtos,$obj;
}

$text = <$arqCompra>;
while($text = <$arqCompra>){
	chomp($text);
	my @lines = split(";", $text);
	last if($text eq "");
	$obj = Compra->new(@lines);
	push @compras,$obj;
}

$text = <$arqForn>;
while($text = <$arqForn>){
	chomp($text);
	my @lines = split(";", $text);
	last if($text eq "");
	$obj = Fornecedor->new(@lines);
	push @fornecedores,$obj;
}

$text = <$arqVenda>;
while($text = <$arqVenda>){
	chomp($text);
	my @lines = split(";", $text);
	last if($text eq "");
	if ($lines[4] ne "F"){ $lines[0] = 0 };
	$obj = Venda->new(@lines);
	push @vendas,$obj;
}

$text = <$arqCliente>;
while($text = <$arqCliente>){
	chomp($text);
	my @lines = split(";", $text);
	last if($text eq "");
	$obj = Cliente->new(@lines);
	push @clientes,$obj;
}

geraSaida;
